# -*- coding: utf-8 -*-

"""Package for abstract_scorm_xblock."""

__author__ = "Alexander Marenich, Andrey Kryachko, Andrey Lykhoman, COTOHA, David Baumgold, Volodymyr Bergman, Maxim Starodubcev, Jorge Mora, Silvio Tomatis, kirkerafael, Oksana Slusarenro, Chiruzzi Marco"
__maintainer__ = "Silvio Tomatis, Chiruzzi Marco"
__email__ = "silviot@gmail.com, chiruzzi.marco@gmail.com"
__version__ = "1.0.0"



# -*- coding: utf-8 -*-
import pkg_resources

from django.template import Context, Template


def gettext(text):
    """Dummy `gettext` replacement to make string extraction
    tools scrape strings marked for translation """
    return text


def resource_string(path):
    """Handy helper for getting resources from our kit."""
    return pkg_resources.resource_string(__name__, path).decode("utf8")


def render_template(template_path, context):
    template_str = resource_string(template_path)
    template = Template(template_str)
    return template.render(Context(context))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.apps import AppConfig
from openedx.core.djangoapps.plugins.constants import PluginSettings
from openedx.core.djangoapps.plugins.constants import PluginURLs
from openedx.core.djangoapps.plugins.constants import ProjectType
from openedx.core.djangoapps.plugins.constants import SettingsType


class AbstractScormXBlockAppConfig(AppConfig):
    name = "abstract_scorm_xblock"
    plugin_app = {
        PluginURLs.CONFIG: {
            ProjectType.LMS: {
                PluginURLs.NAMESPACE: "abstract_scorm_xblock",
                PluginURLs.REGEX: "abstract_scorm_xblock/",
                PluginURLs.RELATIVE_PATH: "views",
            },
            ProjectType.CMS: {
                PluginURLs.NAMESPACE: "abstract_scorm_xblock",
                PluginURLs.REGEX: "abstract_scorm_xblock/",
                PluginURLs.RELATIVE_PATH: "views",
            },
        },
        PluginSettings.CONFIG: {
            ProjectType.LMS: {
                SettingsType.PRODUCTION: {PluginSettings.RELATIVE_PATH: "app"},
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "app"},
                SettingsType.DEVSTACK: {PluginSettings.RELATIVE_PATH: "app"},
                SettingsType.TEST: {PluginSettings.RELATIVE_PATH: "app"},
            },
            ProjectType.CMS: {
                SettingsType.PRODUCTION: {PluginSettings.RELATIVE_PATH: "app"},
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "app"},
                SettingsType.DEVSTACK: {PluginSettings.RELATIVE_PATH: "app"},
                SettingsType.TEST: {PluginSettings.RELATIVE_PATH: "app"},
            },
        },
    }


def plugin_settings(settings):
    if not getattr(settings, "STORAGE_SCORM_PATH", None):
        # This is used to build the path to the location where the
        # SCORM packages are extracted into the default storage
        # e.g. /scorm_packages/09c1735eaa57d78fe245868f0e07cf7b/index_lms.html
        settings.STORAGE_SCORM_PATH = "scorm_packages"



# -*- coding: utf-8 -*-
import json

import mock
import unittest

import ddt

from xblock.field_data import DictFieldData

from .constants import ScormVersions
from .scormxblock import AbstractScormXBlock


@ddt.ddt
class AbstractScormXBlockTests(unittest.TestCase):
    def make_one(self, **kwargs):
        """
        Creates a AbstractScormXBlock for testing purpose.
        """
        field_data = DictFieldData(kwargs)
        xblock = AbstractScormXBlock(mock.Mock(), field_data, mock.Mock())
        xblock.location = mock.Mock(
            block_id="block_id", org="org", course="course", block_type="block_type"
        )
        return xblock

    def test_fields_xblock(self):
        xblock = self.make_one()
        self.assertEqual(xblock.display_name, "SCORM unit")
        self.assertEqual(xblock.scorm_file, "")
        self.assertEqual(xblock.scorm_index, "")
        self.assertEqual(xblock.lesson_score, 0)
        self.assertEqual(xblock.has_score, True)
        self.assertEqual(xblock.icon_class, "problem")
        self.assertEqual(xblock.width, None)
        self.assertEqual(xblock.height, 450)
        self.assertEqual(xblock.popup, False)
        self.assertEqual(xblock.autoopen, False)
        self.assertEqual(xblock.allowopeninplace, False)
        self.assertEqual(xblock.weight, 1)
        self.assertEqual(xblock._scorm_version, ScormVersions["SCORM_12"].value)
        self.assertEqual(xblock._lesson_status, "not attempted")
        self.assertEqual(xblock._success_status, "unknown")

    def test_studio_submit(self):
        xblock = self.make_one()

        fields = {
            "display_name": "Test SCORM XBlock",
            "has_score": "True",
            "icon_class": "video",
            "height": 500,
        }

        xblock.studio_submit(mock.Mock(method="POST", params=fields))
        self.assertEqual(xblock.display_name, fields["display_name"])
        self.assertEqual(xblock.has_score, fields["has_score"])
        self.assertEqual(xblock.icon_class, "video")
        self.assertEqual(xblock.width, None)
        self.assertEqual(xblock.height, 500)

    @mock.patch(
        "abstract_scorm_xblock.scormxblock.AbstractScormXBlock._get_completion_status",
        return_value="completion_status",
    )
    @mock.patch("abstract_scorm_xblock.scormxblock.AbstractScormXBlock._publish_grade")
    @ddt.data(
        {"name": "cmi.core.lesson_status", "value": "completed"},
        {"name": "cmi.completion_status", "value": "failed"},
        {"name": "cmi.success_status", "value": "unknown"},
    )
    def test_set_status(self, value, _publish_grade, _get_completion_status):
        xblock = self.make_one(has_score=True)

        response = xblock.scorm_set_value(
            mock.Mock(method="POST", body=json.dumps(value))
        )

        _publish_grade.assert_called_once_with()
        _get_completion_status.assert_called_once_with()

        if value["name"] == "cmi.success_status":
            self.assertEqual(xblock._success_status, value["value"])
        else:
            self.assertEqual(xblock._lesson_status, value["value"])

        self.assertEqual(
            response.json,
            {
                "completion_status": "completion_status",
                "lesson_score": 0,
                "result": "success",
            },
        )

    @mock.patch(
        "abstract_scorm_xblock.scormxblock.AbstractScormXBlock._get_completion_status",
        return_value="completion_status",
    )
    @ddt.data(
        {"name": "cmi.core.score.raw", "value": "20"},
        {"name": "cmi.score.raw", "value": "20"},
    )
    def test_set_lesson_score(self, value, _get_completion_status):
        xblock = self.make_one(has_score=True)

        response = xblock.scorm_set_value(
            mock.Mock(method="POST", body=json.dumps(value))
        )

        _get_completion_status.assert_called_once_with()

        self.assertEqual(xblock.lesson_score, 0.2)

        self.assertEqual(
            response.json,
            {
                "completion_status": "completion_status",
                "lesson_score": 0.2,
                "result": "success",
            },
        )

    @mock.patch(
        "abstract_scorm_xblock.scormxblock.AbstractScormXBlock._get_completion_status",
        return_value="completion_status",
    )
    @ddt.data(
        {"name": "cmi.core.lesson_location", "value": 1},
        {"name": "cmi.location", "value": 2},
        {"name": "cmi.suspend_data", "value": [1, 2]},
    )
    def test_set_other_scorm_values(self, value, _get_completion_status):
        xblock = self.make_one(has_score=True)

        response = xblock.scorm_set_value(
            mock.Mock(method="POST", body=json.dumps(value))
        )

        _get_completion_status.assert_called_once_with()

        self.assertEqual(xblock._scorm_data[value["name"]], value["value"])

        self.assertEqual(
            response.json,
            {"completion_status": "completion_status", "result": "success"},
        )

    @ddt.data(
        {"name": "cmi.core._lesson_status"},
        {"name": "cmi.completion_status"},
        {"name": "cmi.success_status"},
    )
    def test_scorm_get_status(self, value):
        xblock = self.make_one(_lesson_status="status", _success_status="status")

        response = xblock.scorm_get_value(
            mock.Mock(method="POST", body=json.dumps(value))
        )

        self.assertEqual(response.json, {"value": "status"})

    @ddt.data(
        {"name": "cmi.core.score.raw"}, {"name": "cmi.score.raw"},
    )
    def test_scorm_get_lesson_score(self, value):
        xblock = self.make_one(lesson_score=0.2)

        response = xblock.scorm_get_value(
            mock.Mock(method="POST", body=json.dumps(value))
        )

        self.assertEqual(response.json, {"value": 20})

    @ddt.data(
        {"name": "cmi.core.lesson_location"},
        {"name": "cmi.location"},
        {"name": "cmi.suspend_data"},
    )
    def test_get_other_scorm_values(self, value):
        xblock = self.make_one(
            _scorm_data={
                "cmi.core.lesson_location": 1,
                "cmi.location": 2,
                "cmi.suspend_data": [1, 2],
            }
        )

        response = xblock.scorm_get_value(
            mock.Mock(method="POST", body=json.dumps(value))
        )

        self.assertEqual(response.json, {"value": xblock._scorm_data[value["name"]]})



# -*- coding: utf-8 -*-
import os
import io
import json
import logging
from zipfile import ZipFile

from webob import Response
from lxml import etree

from django.conf import settings
from django.urls import reverse
from django.core.files.storage import default_storage

from xmodule.contentstore.django import contentstore

from xblock.core import XBlock
from xblock.fields import Scope, String, Float, Boolean, Dict, Integer
from xblock.fragment import Fragment

from .utils import gettext as _
from .utils import resource_string, render_template
from .constants import ScormVersions
from .exceptions import ScormManifestNotFoundException, ScormPackageNotFoundException


logger = logging.getLogger(__name__)


class AbstractScormXBlock(XBlock):

    display_name = String(
        display_name=_("Display Name"),
        help=_("Display name for this module"),
        default="SCORM unit",
        scope=Scope.settings,
    )
    scorm_file = String(
        display_name=_("SCORM file package"),
        help=_(
            'Name of the SCORM Zip file uploaded through the "Files & Uploads" section of the Course'
        ),
        default="",
        scope=Scope.settings,
    )
    scorm_index = String(
        display_name=_("SCORM index"),
        help=_(
            "Path to the HTML index page in the SCORM package. If left blank we'll try to guess it's location from the SCORM manifest file"
        ),
        default="",
        scope=Scope.settings,
    )
    has_score = Boolean(
        display_name=_("Scored"),
        help=_(
            "Select False if this unit will not receive a numerical score from the SCORM"
        ),
        default=True,
        scope=Scope.settings,
    )
    icon_class = String(
        display_name=_("Icon class"),
        help=_(
            "The icon which will be displayed on the units navigation in courseware"
        ),
        values=[
            {"display_name": "Problem", "value": "problem"},
            {"display_name": "Video", "value": "video"},
            {"display_name": "Other", "value": "other"},
        ],
        default="problem",
        scope=Scope.settings,
    )
    width = Integer(
        display_name=_("Display Width (px)"),
        help=_("Width of the iframe. If empty defaults to 100%"),
        scope=Scope.settings,
    )
    height = Integer(
        display_name=_("Display Height (px)"),
        help=_("Height of iframe. If empty defaults to 450px"),
        default=450,
        scope=Scope.settings,
    )
    popup = Boolean(
        display_name=_("Popup"),
        help=_("Open in a new popup window, or an iframe."),
        default=False,
        scope=Scope.settings,
    )
    autoopen = Boolean(
        display_name=_("Popup on entry"),
        help=_("Should the popup window show on load (instead of on button click)."),
        default=False,
        scope=Scope.settings,
    )
    allowopeninplace = Boolean(
        display_name=_("Allow open in place"),
        help=_("Show button for opening not-in-popup."),
        default=False,
        scope=Scope.settings,
    )
    weight = Float(default=1, scope=Scope.settings)
    lesson_score = Float(scope=Scope.user_state, default=0)
    _scorm_url = String(
        display_name=_("SCORM file URL"), default="", scope=Scope.settings,
    )
    _scorm_version = String(
        default=ScormVersions["SCORM_12"].value, scope=Scope.settings
    )
    # save completion_status for SCORM_2004
    _lesson_status = String(scope=Scope.user_state, default="not attempted")
    _success_status = String(scope=Scope.user_state, default="unknown")
    _scorm_data = Dict(scope=Scope.user_state, default={})

    def student_view(self, context={}):
        # TODO: We should be able to display an error message
        # instead of trying to render an inexistent or problematic
        # SCORM package
        try:
            self._ensure_scorm_package_is_extracted()
        except ScormManifestNotFoundException as e:
            logger.error(e)
        except ScormPackageNotFoundException as e:
            logger.error(e)

        template = render_template(
            "static/html/scormxblock.html",
            {"completion_status": self._get_completion_status(), "scorm_xblock": self},
        )
        fragment = Fragment(template)
        fragment.add_css(resource_string("static/css/scormxblock.css"))
        fragment.add_javascript(resource_string("static/js/src/scormxblock.js"))
        js_settings = {
            "scorm_version": self._scorm_version,
            "scorm_url": self._scorm_url,
            "completion_status": self._get_completion_status(),
            "scorm_xblock": {
                "display_name": self.display_name,
                "width": self.width,
                "height": self.height,
                "popup": self.popup,
                "autoopen": self.autoopen,
                "allowopeninplace": self.allowopeninplace,
            },
        }
        fragment.initialize_js("ScormXBlock", json_args=js_settings)
        return fragment

    def studio_view(self, context={}):
        # TODO: We should be able to display an error message
        # instead of trying to render an inexistent or problematic
        # SCORM package
        try:
            self._ensure_scorm_package_is_extracted()
        except ScormManifestNotFoundException as e:
            logger.error(e)
        except ScormPackageNotFoundException as e:
            logger.error(e)

        template = render_template(
            "static/html/studio.html",
            {
                "display_name_field": self.fields["display_name"],
                "scorm_file_field": self.fields["scorm_file"],
                "scorm_index_field": self.fields["scorm_index"],
                "has_score_field": self.fields["has_score"],
                "icon_class_field": self.fields["icon_class"],
                "width_field": self.fields["width"],
                "height_field": self.fields["height"],
                "popup_field": self.fields["popup"],
                "autoopen_field": self.fields["autoopen"],
                "allowopeninplace_field": self.fields["allowopeninplace"],
                "scorm_xblock": self,
            },
        )
        fragment = Fragment(template)
        fragment.add_css(resource_string("static/css/scormxblock.css"))
        fragment.add_javascript(resource_string("static/js/src/studio.js"))
        fragment.initialize_js("ScormStudioXBlock")
        return fragment

    @XBlock.handler
    def studio_submit(self, request, suffix=""):
        """
        This function handles the Studio submit AJAX call.
        It must return a WebOb Response object as specified by the XBlock API
        specifications.
        https://edx.readthedocs.io/projects/xblock/en/latest/xblock.html#xblock.core.XBlock.handler
        """
        self.display_name = request.params.get("display_name")
        self.width = request.params.get("width")
        self.height = request.params.get("height")
        self.has_score = request.params.get("has_score")
        self.popup = request.params.get("popup")
        self.autoopen = request.params.get("autoopen")
        self.allowopeninplace = request.params.get("allowopeninplace")
        self.icon_class = request.params.get("icon_class")
        self.scorm_index = request.params.get("scorm_index") or ""
        self.scorm_file = request.params.get("scorm_file")

        try:
            self._save_scorm_package()
        except ScormManifestNotFoundException:
            return Response(
                json.dumps({"field": "scorm_file", "message": "Invalid SCORM file"}),
                content_type="application/json",
                status=400,
            )
        except ScormPackageNotFoundException:
            return Response(
                json.dumps(
                    {"field": "scorm_file", "message": "SCORM package not found"}
                ),
                content_type="application/json",
                status=404,
            )

        return Response(
            json.dumps({"message": "XBlock saved successfully !"}),
            content_type="application/json",
            status=200,
        )

    @XBlock.json_handler
    def scorm_get_value(self, data, suffix=""):
        name = data.get("name")
        if name in ["cmi.core._lesson_status", "cmi.completion_status"]:
            return {"value": self._lesson_status}
        elif name == "cmi.success_status":
            return {"value": self._success_status}
        elif name in ["cmi.core.score.raw", "cmi.score.raw"]:
            return {"value": self.lesson_score * 100}
        else:
            return {"value": self._scorm_data.get(name, "")}

    @XBlock.json_handler
    def scorm_set_value(self, data, suffix=""):
        payload = {"result": "success"}
        name = data.get("name")

        if name in ["cmi.core.lesson_status", "cmi.completion_status"]:
            self._lesson_status = data.get("value")
            if self.has_score and data.get("value") in [
                "completed",
                "failed",
                "passed",
            ]:
                self._publish_grade()
                payload.update({"lesson_score": self.lesson_score})
        elif name == "cmi.success_status":
            self._success_status = data.get("value")
            if self.has_score:
                if self._success_status == "unknown":
                    self.lesson_score = 0
                self._publish_grade()
                payload.update({"lesson_score": self.lesson_score})
        elif name in ["cmi.core.score.raw", "cmi.score.raw"] and self.has_score:
            self.lesson_score = int(data.get("value", 0)) / 100.0
            self._publish_grade()
            payload.update({"lesson_score": self.lesson_score})
        else:
            self._scorm_data[name] = data.get("value", "")

        payload.update({"completion_status": self._get_completion_status()})
        return payload

    def _publish_grade(self):
        if self._lesson_status == "failed" or (
            ScormVersions(self._scorm_version) > ScormVersions["SCORM_12"]
            and self._success_status in ["failed", "unknown"]
        ):
            self.runtime.publish(self, "grade", {"value": 0, "max_value": self.weight})
        else:
            self.runtime.publish(
                self, "grade", {"value": self.lesson_score, "max_value": self.weight}
            )

    def _get_completion_status(self):
        completion_status = self._lesson_status
        if (
            ScormVersions(self._scorm_version) > ScormVersions["SCORM_12"]
            and self._success_status != "unknown"
        ):
            completion_status = self._success_status
        return completion_status

    def _read_scorm_manifest(self, scorm_path):
        manifest_path = os.path.join(scorm_path, "imsmanifest.xml")
        try:
            return default_storage.open(manifest_path).read()
        except IOError:
            raise ScormManifestNotFoundException()

    def _update_scorm_url(self, scorm_md5):
        # We can't load the scorm file directly from the default_storage URL
        # since it will be blocked by the SAMEORIGIN policy
        self._scorm_url = reverse(
            "abstract_scorm_xblock:scorm_serve",
            kwargs={"md5": scorm_md5, "path": self.scorm_index},
        )

    def _update_scorm_index(self, manifest):
        if not self.scorm_index:
            try:
                self.scorm_index = (
                    etree.fromstring(manifest).find(".//{*}resource").get("href")
                )
            except (IOError, AttributeError):
                self.scorm_index = "index.html"

    def _update_scorm_version(self, manifest):
        try:
            self._scorm_version = (
                etree.fromstring(manifest).find(".//{*}schemaversion").text
            )
        except (IOError, AttributeError):
            self._scorm_version = ScormVersions["SCORM_12"].value

    def _search_scorm_package(self):
        scorm_content, count = contentstore().get_all_content_for_course(
            self.course_id,
            filter_params={
                "contentType": "application/zip",
                "displayname": self.scorm_file,
            },
        )
        if not count:
            raise ScormPackageNotFoundException()
        # Since course content names are unique we are sure that we
        # can't have multiple results, so we just pop the first.
        return scorm_content.pop()

    def _extract_scorm_package(self, scorm_package):
        """
        Extracts the SCORM package to the default storage if needed
        """
        scorm_path = os.path.join(settings.STORAGE_SCORM_PATH, scorm_package["md5"])
        if not default_storage.exists(scorm_path):
            # We are actually loading the whole zipfile in memory.
            # This step should probably be handled more carefully.
            scorm_zipfile_data = contentstore().find(scorm_package["asset_key"]).data
            with ZipFile(io.BytesIO(scorm_zipfile_data)) as zipfile_obj:
                # We can't save extracted files directly
                # ZipFile does not provide a seek method till python 3.7
                # https://bugs.python.org/issue22908
                for filename in zipfile_obj.namelist():
                    default_storage.save(
                        os.path.join(
                            settings.STORAGE_SCORM_PATH, scorm_package["md5"], filename
                        ),
                        io.BytesIO(zipfile_obj.read(filename)),
                    )
        return scorm_path

    def _ensure_scorm_package_is_extracted(self):
        scorm_package = self._search_scorm_package()
        scorm_path = self._extract_scorm_package(scorm_package)
        return scorm_package, scorm_path

    def _save_scorm_package(self):
        if self.scorm_file:
            scorm_package, scorm_path = self._ensure_scorm_package_is_extracted()
            manifest = self._read_scorm_manifest(scorm_path)
            self._update_scorm_version(manifest)
            self._update_scorm_index(manifest)
            self._update_scorm_url(scorm_package["md5"])
        else:
            self._scorm_url = ""
            self._scorm_version = ""

    @staticmethod
    def workbench_scenarios():
        """A canned scenario for display in the workbench."""
        return [
            (
                "AbstractScormXBlock",
                """<vertical_demo>
                <abstract_scorm_xblock/>
                </vertical_demo>
            """,
            ),
        ]



# -*- coding: utf-8 -*-
from enum import Enum


ORDERED_VERSIONS = [
    "SCORM_12",
    "SCORM_2004_2_EDITION",
    "SCORM_2004_3_EDITION",
    "SCORM_2004_4_EDITION",
]


class ScormVersions(Enum):
    """
    List SCORM versions which can be found in supported manifest templates.
    Reference templates can be found at https://scorm.com/scorm-explained/technical-scorm/content-packaging/xml-schema-definition-files/
    """

    SCORM_12 = "1.2"
    SCORM_2004_2_EDITION = "CAM 1.3"
    SCORM_2004_3_EDITION = "2004 3rd Edition"
    SCORM_2004_4_EDITION = "2004 4th Edition"

    def __lt__(self, other):
        assert isinstance(other, self.__class__)
        return ORDERED_VERSIONS.index(self.name) < ORDERED_VERSIONS.index(other.name)

    def __gt__(self, other):
        assert isinstance(other, self.__class__)
        return ORDERED_VERSIONS.index(self.name) > ORDERED_VERSIONS.index(other.name)



# -*- coding: utf-8 -*-
"""Setup for abstract_scorm_xblock XBlock."""

import io
from setuptools import setup

with io.open("README.md", encoding="utf-8") as readme_file:
    readme = readme_file.read()

setup(
    name="abstract-scorm-xblock",
    version="1.0.0",
    description="Load SCORM packages into Open edX courses",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=["abstract_scorm_xblock"],
    install_requires=["XBlock"],
    include_package_data=True,
    keywords=["scorm", "xblock"],
    author="Alexander Marenich, Andrey Kryachko, Andrey Lykhoman, COTOHA, David Baumgold, Volodymyr Bergman, Maxim Starodubcev, Jorge Mora, Silvio Tomatis, kirkerafael, Oksana Slusarenro, Chiruzzi Marco",
    author_email="sendr84@gmail.com, andrey.kryachko@raccoongang.com, andrey.likhoman@gmail.com, sergiy.movchan@gmail.com, david@davidbaumgold.com, wowkalucky@gmail.com, starodubcevmax@gmail.com, jorge.mora@innovapues.com, silviot@gmail.com, kirkerafael@gmail.com, oksana.slu@gmail.com, chiruzzi.marco@gmail.com",
    maintainer="Silvio Tomatis, Chiruzzi Marco",
    maintainer_email="silviot@gmail.com, chiruzzi.marco@gmail.com",
    url="https://github.com/Abstract-Tech/abstract-scorm-xblock",
    download_url="https://github.com/Abstract-Tech/abstract-scorm-xblock/archive/v1.0.0.tar.gz",
    license="Apache 2.0",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Framework :: Django",
        "License :: OSI Approved :: Apache Software License",
    ],
    entry_points={
        "xblock.v1": [
            "abstract_scorm_xblock = abstract_scorm_xblock.scormxblock:AbstractScormXBlock"
        ],
        "lms.djangoapp": [
            "abstract_scorm_xblock = abstract_scorm_xblock.app:AbstractScormXBlockAppConfig"
        ],
        "cms.djangoapp": [
            "abstract_scorm_xblock = abstract_scorm_xblock.app:AbstractScormXBlockAppConfig"
        ],
    },
)



# -*- coding: utf-8 -*-
import mimetypes
import os
import posixpath

from django.conf import settings
from django.http import FileResponse, Http404

from django.utils.http import http_date
from django.utils.six.moves.urllib.parse import unquote
from django.core.files.storage import default_storage
from django.conf.urls import url


def scormxblock_serve(request, md5, path):
    """
    Proxy files from the Django default storage in order to avoid SAMEORIGIN issues.
    """
    path = posixpath.normpath(unquote(path)).lstrip("/")
    fullpath = os.path.join(settings.STORAGE_SCORM_PATH, md5, path)

    if not default_storage.exists(fullpath):
        raise Http404()

    content_type, encoding = mimetypes.guess_type(fullpath)
    content_type = content_type or "application/octet-stream"
    response_file = default_storage.open(fullpath)
    response = FileResponse(response_file, content_type=content_type)
    response["Last-Modified"] = http_date(
        default_storage.get_modified_time(fullpath).toordinal()
    )
    response["Content-Length"] = response_file.size
    if encoding:
        response["Content-Encoding"] = encoding
    return response


urlpatterns = [
    url(r"(?P<md5>[a-f0-9]{32})/(?P<path>.*)$", scormxblock_serve, name="scorm_serve")
]
